import tkinter as tk
from tkinter.filedialog import askopenfilename, asksaveasfilename


class UItextEditor(tk.Tk):

    def __init__(self):
        super().__init__()

        self.title("Ghetto Recorder Editor")
        self.rowconfigure(0, minsize=400, weight=1)
        self.columnconfigure(1, minsize=400, weight=1)

        self.fr_buttons = tk.Frame(self, relief=tk.RAISED, bd=2)
        self.fr_buttons.grid(row=0, column=0, sticky="ns")
        self.txt_edit = tk.Text(self)
        self.txt_edit.grid(row=0, column=1, sticky="nsew")

        self.btn_open = tk.Button(self.fr_buttons, text="Open", command=self.open_file)
        self.btn_save = tk.Button(self.fr_buttons, text="Save As...", command=self.save_file)

        self.btn_open.grid(row=0, column=0, sticky="ew", padx=5, pady=5)
        self.btn_save.grid(row=1, column=0, sticky="ew", padx=5)

    def open_file(self):
        filepath = askopenfilename(
            filetypes=[("INI Files", "*.ini"), ("All Files", "*.*")]
        )
        if not filepath:
            return
        self.txt_edit.delete(1.0, tk.END)
        with open(filepath, "r") as input_file:
            text = input_file.read()
            self.txt_edit.insert(tk.END, text)
        self.title(f"Ghetto Recorder Editor - {filepath}")

    def save_file(self):
        filepath = asksaveasfilename(
            defaultextension="txt",
            filetypes=[("INI Files", "*.ini"), ("All Files", "*.*")],
        )
        if not filepath:
            return
        with open(filepath, "w") as output_file:
            text = self.txt_edit.get(1.0, tk.END)
            output_file.write(text)
        self.title(f"Ghetto Recorder Editor - {filepath}")



import logging
from pathlib import Path
import os
from tkinter import ttk
from time import sleep
from ghettorecorder.lib.edit import UItextEditor
from ghettorecorder.lib import term
try:
    import tkinter as tk
except ImportError:
    print(f'\n\n  Please install --> python3-tk: on Ubuntu sudo apt-get install python3-tk\n\n')
try:
    from tkinter.filedialog import askdirectory, askopenfilename
except ImportError:
    print(f'\n error tkinter.filedialog import askdirectory, askopenfilename\n')

# logging.basicConfig(level=logging.DEBUG, format='(%(threadName)-10s) %(message)s',)


class UIWindow(tk.Tk):

    box_list = []
    search_list = []
    entry_list = []
    lbl_li = []

    timer_val = ['', 1, 2, 4, 6, 12, 24]
    data_dir_set = False
    record_started = False
    search_started = False

    info_text = [
        'Die Zeilen in der Mitte sind für Suchbegriffe da.',
        'Man sollte vorher im linken Kasten den Haken setzen. "Search"',
        'Begriffe wie: Elvis mozart band Beton, in die Zeile schreiben',
        'Es ist egal, ob das Wort GROSS oder klein geschrieben ist.',
        'Das Radio spielt den Titel mit dem Wort, Aufnahme beginnt.',
        'Bei schnellen Titelwechseln kann noch ein Teil des folgenden',
        'Titels aufgenommen werden. Einfach löschen.',
        '"Timer" oben rechts, wenn man nicht selber Stop drücken will.',
        'Der Timer ist die Eieruhr. nur in Stunden. Fertig! :)',
        'Die Uhr kann man jederzeit ändern. Viel Spaß!',
    ]

    info_te_eng = [
        'The lines in the middle area are used for searching phrases.',
        'Please activate the corresponding box on the left. "Search"',
        'Phrases like: Elvis mozart band concrete, can be written.',
        'There is no rule for writing the words UPPER or lower case.',
        'Radio station is playing a title with a search phrase! Record.',
        'Sometimes titles are published before or after the content',
        'changes. Get rid of the garbage files. Delete it.',
        'A "Timer" in the upper right corner. If you do not like pressing',
        'the "Stop" button yourself. One can change the value every time.',
        'Have fun!'
    ]

    def __init__(self, radio_dir=None, settings_path=None):
        super().__init__()
        self.gr_version = '0.1.1'
        self.radio_dir = radio_dir
        self.settings_path = settings_path

        self.title(f"Ghetto Recorder {self.gr_version} Concrete IT (by René Horn)")
        self.geometry('{}x{}'.format(620, 460))

        # main containers create
        self.top_frame = tk.Frame(master=self, bg='azure3', width=600, height=50, pady=3)
        self.center = tk.Frame(master=self, bg='gray2', width=50, height=40, padx=3, pady=3)
        self.info = tk.Frame(master=self, bg='azure3', width=50, height=2, padx=1, pady=1)
        self.btm_frame = tk.Frame(master=self, bg='azure3', width=600, height=40, pady=3)

        # main containers layout
        self.grid_rowconfigure(1, weight=1)
        self.grid_columnconfigure(0, weight=1)

        self.top_frame.grid(row=0, sticky="ew")
        self.center.grid(row=1, sticky="nsew")
        self.info.grid(row=2, sticky="nsew")
        self.btm_frame.grid(row=3, sticky="ew")

        # center widgets create
        self.center.grid_rowconfigure(0, weight=1)
        self.center.grid_columnconfigure(1, weight=1)

        self.ctr_left = tk.Frame(self.center, bg='blue', width=25, height=190)
        self.canvas = tk.Canvas(self.center, bg='cornflowerblue', width=250, height=190, bd=0)
        self.ctr_mid = tk.Frame(self.center, bg='azure3', width=250, height=190, padx=3, pady=3)
        self.ctr_right = tk.Frame(self.center, bg='green', width=25, height=190, padx=3, pady=3)
        # center grid
        self.ctr_left.grid(row=0, column=0, sticky="ns")
        self.canvas.grid(row=0, column=1, sticky="nsew")

        self.ctr_mid.grid(row=0, column=1, sticky="nsew")
        self.ctr_right.grid(row=0, column=2, sticky="ns")
        self.ctr_mid.grid_remove()  # call .grid again

        # upper frame Buttons
        self.lbl_ini = tk.Label(self.top_frame, bg='azure3')
        self.btn_ini = tk.Button(self.top_frame, text="Radio list", width=10, command=self.browse_file_button)
        self.lbl_browse = tk.Label(self.top_frame, bg='azure3')
        self.btn_browse = tk.Button(self.top_frame, text="Save to ...", width=10, command=self.browse_dir_button)
        self.lbl_timer = tk.Label(self.top_frame, text='Timer', bg='azure3', anchor="s")
        self.combo_timer = ttk.Combobox(self.top_frame, state="readonly", width=5, values=self.timer_val)

        self.entry_ini = tk.Entry(self.top_frame, background="white", borderwidth=1, relief=tk.FLAT, width=70)
        self.entry_ini.insert(0, ' Path to settings.ini')
        self.entry_bbr = tk.Entry(self.top_frame, background="white",  borderwidth=1, relief=tk.FLAT, width=70)
        self.entry_bbr.insert(0, ' place to create a bunch of folders')

        # upper frame layout

        self.lbl_ini.grid(row=0, column=0, padx=2, pady=2, sticky="w")
        self.btn_ini.grid(row=0, column=1, padx=2, pady=2, sticky='w')
        self.entry_ini.grid(row=0, column=2, columnspan=3, padx=5, sticky='e')
        self.lbl_timer.grid(row=0, column=5, padx=10, sticky="s")

        self.lbl_browse.grid(row=1, column=0, padx=2, pady=2, sticky="w")
        self.btn_browse.grid(row=1, column=1, padx=2, sticky='w')
        self.entry_bbr.grid(row=1, column=2, columnspan=3, padx=5, sticky='e')
        self.combo_timer.grid(row=1, column=5, padx=10, sticky='e')

        # center right
        self.progressbar = ttk.Progressbar(self.ctr_right, orient="vertical", length=350, mode="determinate")
        # bottom

        self.lbl_rec = tk.Label(master=self.btm_frame, bg='azure3', anchor="s")
        self.btn_editor = tk.Button(master=self.btm_frame, text="Radio Editor",
                                    width=10,  padx=2, pady=1, command=UItextEditor)
        self.btn_stop = tk.Button(master=self.btm_frame, text="Stop", width=10, padx=2, pady=1, command=self.exit_app)
        self.btn_rec = tk.Button(master=self.btm_frame, text="Record", width=10,
                                 padx=2, pady=1, command=self.list_checked)
        self.lbl_info = tk.Label(master=self.btm_frame, bg='azure3', anchor="s")

        # bottom grid
        self.lbl_rec.grid(row=0, column=0, padx=10, sticky="w")
        self.btn_editor.grid(row=0, column=1, sticky='e')
        self.btn_stop.grid(row=0, column=2, sticky='w')
        self.btn_rec.grid(row=0, column=3, sticky='w')
        self.lbl_info.grid(row=0, column=4, sticky='e')

        self.canvas.create_rectangle(0, 0, 250, 190, fill='cornflowerblue', width=0)
        self.canvas.create_line(0, 125, 2000, 125, fill='orange', width=10)
        self.canvas.bind("<Enter>", self.on_enter)
        self.canvas.bind("<Leave>", self.on_leave)

        # show path to settings.ini
        ini_file = os.path.join(os.path.dirname(os.path.abspath(__file__)), "settings.ini")
        term.GBase.settings_path = ini_file
        self.entry_ini.delete(0, tk.END)
        self.entry_ini.insert(0, ini_file)
        self.lbl_info.config(text='Oben links der Knopf ist für die Datei mit den Web Radios.')

    def on_enter(self, event):
        self.canvas.create_line(0, 225, 2000, 225, fill='orange', width=10, tag="line")

    def on_leave(self, enter):
        self.canvas.delete("line")

    def set_settings_path(self, settings_path):
        self.settings_path = settings_path

    def get_settings_path(self):
        return self.settings_path

    def set_radio_dir(self, radio_dir):
        self.radio_dir = radio_dir

    def get_radio_dir(self):
        return self.radio_dir

    def browse_dir_button(self):
        dir_name = askdirectory(title='Ghetto Recorder')
        if not dir_name:
            return
        self.set_radio_dir(dir_name)
        self.entry_bbr.delete(0, tk.END)
        self.entry_bbr.insert(0, dir_name)
        term.GBase.radio_base_dir = dir_name
        # self.lbl_browse.config(text=dir_name)
        self.data_dir_set = True
        return dir_name

    def browse_file_button(self):
        filepath = askopenfilename(
            filetypes=[("INI Files", "*.ini"), ("All Files", "*.*")],
            title='Ghetto Recorder',
            initialdir=os.path.dirname(os.path.abspath(__file__))
        )
        if not filepath:
            return
        self.set_settings_path(filepath)
        self.entry_ini.delete(0, tk.END)
        self.entry_ini.insert(0, filepath)

        term.GBase.settings_path = filepath  # set new path
        term.GIni.show_items_ini_file()
        # get the radios
        self.load_settings(term.GIni.list_items, self.ctr_mid)  # only keys, not values
        self.btn_ini = tk.Button(self.top_frame, text="Radio list", width=10, state="disabled")
        self.btn_ini.grid(row=0, column=1, sticky='w')

        return filepath

    def print_dir(self):
        data_folder = os.path.join(os.path.dirname(os.path.abspath(__file__)), "radiostations")
        test_file = os.path.join(os.path.dirname(os.path.abspath(__file__)), "kreuzberg")
        try:
            with open(test_file, 'wb') as record_file:

                record_file.write(b'\x03')
            os.remove(test_file)

            if not self.data_dir_set:

                term.GBase.radio_base_dir = data_folder
                term.GBase.make_directory(data_folder)
                self.entry_bbr.delete(0, tk.END)
                self.entry_bbr.insert(0, data_folder)

        except Exception as ex:
            print(ex)
            self.entry_bbr.delete(0, tk.END)
            self.entry_bbr.insert(0, 'My directory is read only')

    def exit_app(self):
        term.GBase.exit_app = True
        sleep(term.GBase.sleeper + 1)  # threads sleep too, time for exit
        self.btn_rec = tk.Button(master=self.btm_frame, text="Record", width=10, command=self.list_checked)
        self.btn_rec.grid(row=0, column=3, sticky='w')
        term.GBase.exit_app = False

    def cb_checked(self):
        is_playlist_server = ''
        for ctr, int_var in enumerate(self.box_list):
            if int_var.get():     # IntVar not zero==checked
                self.lbl_li[ctr].configure(foreground='black')  # <widget>.configure(foreground='black',bg='red')

    def cb_search(self):
        for ctr, int_var in enumerate(self.search_list):
            # search checkbox selection
            if int_var.get():  # IntVar not zero==checked
                self.entry_list[ctr].delete(0, tk.END)
                # set the related record check box
                self.box_list[ctr].set(1)
                self.lbl_li[ctr].configure(foreground='red')  # <widget>.configure(foreground='black',bg='red')
                print(self.lbl_li[ctr].cget("text"))

    def load_settings(self, list_of_widgets, frame):

        self.print_dir()
        self.box_list = []
        self.search_list = []
        # remove start screen
        self.ctr_mid.grid()
        self.canvas.grid_remove()
        tk.Label(frame, text='Record', underline=0 , bg='azure3').grid(row=0, column=2, sticky="e")
        tk.Label(frame, text='Add search (Record if match - multi): podcast concert elvis', underline=0,
                 bg='azure3').grid(row=0, column=1, sticky="n")
        tk.Label(frame, text='Search', bg='azure3').grid(row=0, column=0, sticky="w")
        tk.Label(frame, text='Radio', bg='azure3').grid(row=0, column=3, sticky="w")

        term.GBase.pool.submit(self.display_info_text)

        for idx, text in enumerate(list_of_widgets):

            self.box_list.append(tk.IntVar())  # auto var from python PY_VAR0 1 2 for finding check buttons
            self.search_list.append(tk.IntVar())
            lbl = tk.Label(frame, text=text, bg='azure3')
            lbl.grid(row=idx + 1, column=3, sticky="w")
            self.lbl_li.append(lbl)

            entry = tk.Entry(frame, width=60, bg='azure3')
            entry.grid(row=idx + 1, column=1, sticky="w")

            self.entry_list.append(entry)
            tk.Checkbutton(frame, variable=self.search_list[-1],
                           command=self.cb_search, bg='azure3').grid(row=idx + 1, column=0, sticky='e')

            tk.Checkbutton(frame, variable=self.box_list[-1],
                           command=self.cb_checked, bg='azure3').grid(row=idx + 1, column=2, sticky='e')

    def list_checked(self):

        for ctr, int_var in enumerate(self.box_list):
            # all checked rec.
            if int_var.get():

                # all get a string, that is never found
                term.GIni.search_dict[self.lbl_li[ctr].cget("text")] = "x!c?42"

                # SEARCH check only metadata, start rec. if found
                self.record_started = True
                if self.search_list[ctr].get():  # not zero = True
                    self.search_started = True
                    # update dict for threads going to search def GRecorder.search_pattern_start_record
                    term.GIni.search_dict[self.lbl_li[ctr].cget("text")] = self.entry_list[ctr].get()
                    term.GIni.search_title_keys_list.append(self.lbl_li[ctr].cget("text"))
                    sleep(2)
                    # update label with search content from entry
                    self.lbl_info.config(text=self.entry_list[ctr].get())

                # PLAYLIST is handled in add_server_to..UI, bit diff. than in terminal
                self.add_server_to_data_base_ui(str(self.lbl_li[ctr].cget("text")),
                                                str(term.GIni.find_ini_file(self.lbl_li[ctr].cget("text"))))

        self.btn_rec = tk.Button(master=self.btm_frame, text="Record", width=10, state="disabled")
        self.btn_rec.grid(row=0, column=3, sticky='w')

        # here updated list, zombies deleted
        for key in term.GIni.ini_keys:
            term.test_stream_server(key)

        for key in term.GIni.ini_keys:
            term.GBase.pool.submit(self.start_records, key)

        term.GBase.pool.submit(self.display_title)

        # if selected, start timer
        if self.combo_timer.get():
            new_timer = int(self.combo_timer.get()) * 3600  # 60s * 60m = ?
            term.GBase.pool.submit(self.timer, new_timer)
            self.progressbar.grid(row=0, column=0, sticky='s')

    @staticmethod
    def start_records(key):

        # ----- no del -- term.GRecorder.thread_pull_song_name(url, key, None, None)
        stream_suffix = term.GIni.srv_param_dict[key + '_file']
        term.GIni.song_dict[key] = str('_no-name-record_no_split_')  # init the dict for this thread
        term.GIni.start_stop_recording[key] = 'start'
        term.GIni.start_stop_recording[key + '_adv'] = 'start_from_here'

        for _ in term.GIni.search_title_keys_list:
            if _ == key:
                term.GIni.start_stop_recording[key] = 'stop'

        url = term.GIni.ini_keys[key]
        print(f'{key} {url}')
        dir_save = term.GBase.radio_base_dir + '//' + key
        term.GBase.pool.submit(term.GRecorder.record_songs, url, dir_save, stream_suffix, key, None)
        term.GRecorder.path_to_song_dict = term.GIni.song_dict
        term.GBase.pool.submit(term.GRecorder.thread_pull_song_name, url, key, None, None)

    def display_title(self):

        while not term.GBase.exit_app:
            for idx, var in enumerate(self.box_list):
                if var.get():
                    radio = str(self.lbl_li[idx].cget("text"))
                    self.entry_list[idx].delete(0, tk.END)
                    try:
                        title = term.GRecorder.path_to_song_dict[radio]
                        self.entry_list[idx].insert(0, title)
                    except Exception as ex:
                        print(ex)

            for sec in range(10):
                sleep(1)
                if term.GBase.exit_app:
                    break

    @staticmethod
    def add_server_to_data_base_ui(str_key, str_val):
        is_playlist_server = ''
        term.GIni.ini_keys[str_key] = str_val  # append url to dictionary as value
        term.GBase.make_directory(term.GBase.radio_base_dir + '//' + str_key)
        # playlist url?
        if str_val[-4:] == '.m3u' or str_val[-4:] == '.pls':  # or url[-5:] == '.m3u8' or url[-5:] == '.xspf':
            # take first from the list
            is_playlist_server = UIUtils.playlist_m3u(str_val)
        if not is_playlist_server == '':  # update dictionary with new url
            term.GIni.ini_keys[str_key] = is_playlist_server  # append dictionary, test if it is alive
            if not term.GNet.is_server_meta_stream(term.GIni.ini_keys[str_key]):
                print('   --> playlist_server server failed, no recording')
                del term.GIni.ini_keys[str_key]
        if not term.GNet.is_server_meta_stream(term.GIni.ini_keys[str_key]):  # first time internet access, response code
            # delete key from dict, return
            del term.GIni.ini_keys[str_key]

    def timer(self, time_left):
        combo_time = 0
        current_timer = 0
        while time_left - current_timer:
            current_timer += 1
            self.progress(current_timer, time_left)
            sleep(1)
            if self.combo_timer.get():
                combo_time = int(self.combo_timer.get()) * 3600
            if not combo_time == time_left:
                time_left = combo_time
            if term.GBase.exit_app:
                break
            if not self.record_started:
                break

        term.GBase.exit_app = True
        sleep(term.GBase.sleeper + 1)  # threads sleep too, time for exit
        term.GBase.exit_app = False
        self.btn_rec = tk.Button(master=self.btm_frame, text="Record", width=10, command=self.list_checked)
        self.btn_rec.grid(row=0, column=3, sticky='w')

    def progress(self, current_timer, max_value):
        # progressbar['value'] = 20 , is the percentage from 100, combobox returns string value
        # doing some math, p = (P * 100) / G, percent = (math.percentage value * 100) / base
        cur_percent = round((current_timer * 100) / max_value, 0)
        # print(f'cur_percent {cur_percent}  current_timer {current_timer} max {max_value}')
        self.progressbar['value'] = cur_percent
        self.update_idletasks()
        sleep(1)

    def display_info_text(self):

        while not term.GBase.exit_app:
            if self.search_started or self.record_started:
                self.lbl_info.config(text=' ')
                break
            for line in UIWindow.info_text:
                self.lbl_info.config(text=line)
                for _ in range(10):
                    if self.search_started or self.record_started:
                        self.lbl_info.config(text=' ')
                        break
                    sleep(1)

            for line in UIWindow.info_te_eng:
                self.lbl_info.config(text=line)
                for _ in range(10):
                    if self.search_started or self.record_started:
                        self.lbl_info.config(text=' ')
                        break
                    sleep(1)


class UIUtils(UIWindow):

    def __init__(self, radio_dir=None, settings_path=None):
        super().__init__()

    @staticmethod
    def playlist_m3u(url):
        # returns the first server of the playlist
        try:
            read_url = term.GNet.http_pool.request('GET', url, preload_content=False)
        except Exception as ex:
            print(ex)
        else:
            file = read_url.read().decode('utf-8')

            m3u_lines = file.split("\n")
            # print(' \n    m3u_lines    ' + file)
            m3u_lines = list(filter(None, m3u_lines))  # remove empty rows
            m3u_streams = []
            for row_url in m3u_lines:
                if row_url[0:4].lower() == 'http'.lower():
                    m3u_streams.append(row_url)  # not to lower :)
                    # print(len(m3u_streams))

            if len(m3u_streams) > 1:
                print(' !!! Have more than one stream in playlist_m3u. !!! Take first stream available.')
                play_server = m3u_streams[0]
                return play_server
            if len(m3u_streams) == 1:
                # print(' One server found in playlist_m3u')
                play_server = m3u_streams[0]
                return play_server
            if len(m3u_streams) == 0:
                # print(' No http ... server found in playlist_m3u !!! -EXIT-')
                return False

    @staticmethod
    def sum(arg):
        total = 0
        for val in arg:
            total += val
        return total

if __name__ == "__main__":
    uiw = UIWindow()
    uiw.mainloop()







# 0.0.40
##################################################################################
#   MIT License
#
#   Copyright (c) [2021] [René Horn]
#
#   Permission is hereby granted, free of charge, to any person obtaining a copy
#   of this software and associated documentation files (the "Software"), to deal
#   in the Software without restriction, including without limitation the rights
#   to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
#   copies of the Software, and to permit persons to whom the Software is
#   furnished to do so, subject to the following conditions:
#
#   The above copyright notice and this permission notice shall be included in all
#   copies or substantial portions of the Software.
#
#   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#   AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#   LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
#   OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
#   SOFTWARE.
###################################################################################
import configparser
import io
import json
import logging
import os
import signal
import sys
from concurrent.futures import ThreadPoolExecutor
from time import sleep, strftime, time
import urllib3
##############
import getpass
import shutil
import stat

# logging.basicConfig(level=logging.DEBUG, format='[%(levelname)s] (%(threadName)-10s) %(message)s',)

# Could anyone please explain ... what it does?
# Well, this SCRIPT reads the data stream from a radio station into separate files.
# The data stream is SPLIT INTO PIECES, mostly "ARTIST - Title.MP3" style. Plus ".m3u" and ".pls" play list support.
# 'This' world is tumbling down. Record it with     !!! Ghetto RECORDER !!!
# Be brave, be curious!                                 rene_horn@gmx.net


# Usage:
# Runs on Python 3.5: R E A D --> (Windows 'pip'/Linux 'pip3, python3')
# Go to the next free internet WLAN.
# Windows Store/Linux package manager, install latest Python3 version.
# 'pip install ghettorecorder' - with a normal user account!
# 'pip show ghettorecorder' to find the install Location: site-packages/ghettorecorder /
# 'python - m ghettorecorder.run' will run the recorder from anywhere on your computer.
# GhettoRecorder is uninstalled by 'pip uninstall gettorecorder'. Recorded mp3 files not.
#  Copy 'run.py' and 'settings.ini' files wherever you want to have your record repository.
#  Windows let's you double click 'run.py'. Can run from desktop.
# Magic number 42 is used to connect to all radios found in 'settings.ini'.

os.chdir(os.path.dirname(os.path.abspath(__file__)))  # go to .py script folder
# #####################################################
# username = getpass.getuser()
# # print(username)
# ghetto_folder = '//home//' + username + '//GhettoRecorder'
# source_ini = os.path.dirname(os.path.abspath(__file__)) + '//settings.ini'
# source_script = os.path.dirname(os.path.abspath(__file__)) + '//startGhettoRecorder'
# dst_ini = ghetto_folder + '//settings.ini'
# dst_script = ghetto_folder + '//startGhettoRecorder'
#
# access_rights = 0o755
# try:
#     os.mkdir(ghetto_folder, access_rights)
# except FileExistsError:
#     pass
#
# try:
#     if not os.path.exists(dst_ini):
#         shutil.copyfile(source_ini, dst_ini)
#         shutil.copyfile(source_script, dst_script)
# except FileExistsError:
#     pass
# st = os.stat(dst_script)
# os.chmod(dst_script, st.st_mode | stat.S_IEXEC)
# ##############################################
exit_app = False
urllib3.disable_warnings()
# print(sys.version)

def signal_handler(sig, frame):
    global exit_app
    print(' Stop recording ...')
    exit_app = True
    GBase.exit_app = True
    # sleep(3)  # give thread def: time for clean exit -
    sys.exit()


class GBase:
    # class attribute
    exit_app = False
    sleeper = 2
    pool = ThreadPoolExecutor(200)
    radio_base_dir = os.path.dirname(os.path.abspath(__file__)) + '//radiostations'  # if not set set_radio_base_dir()
    # radio_base_dir = ghetto_folder
    settings_path = os.path.dirname(os.path.abspath(__file__)) + '//settings.ini'  # if not set in set_settings_path()
    # settings_path = radio_base_dir + '//settings.ini'
    path = os.getcwd()
    path_to = path + '//'
    timer = 0

    def __init__(self, radio_base_dir=None, settings_path=None):
        self.instance_attr_time = 0
        self.trigger = False
        self.radio_base_dir = radio_base_dir
        self.settings_path = settings_path

    def set_settings_path(self, settings_path):
        self.settings_path = settings_path

    def get_settings_path(self):
        return self.settings_path

    def set_radio_base_dir(self, radio_base_dir):
        self.radio_base_dir = radio_base_dir

    def get_radio_base_dir(self):
        return self.radio_base_dir

    @staticmethod
    def make_directory(str_path):
        access_rights = 0o755
        try:
            os.mkdir(str_path, access_rights)
        except FileExistsError:
            pass
            #  print(repr(ex))
            # print(' Folder exists: ' + str_path)
            return False
        else:
            print('Successfully created the directory: ' + str_path)
            return True

    @staticmethod
    def remove_special_chars(str_name):
        # cleanup for writing files and folders

        # my_str = "hey th~!ere. /\ coolleagues?! Straße"
        ret_value = str_name.translate({ord(string): "" for string in '"!@#$%^*()[]{};:,./<>?\|`~=+"""'})
        return ret_value

    @staticmethod
    def this_time():
        time = strftime("%Y-%m-%d %H:%M:%S")
        return time

    def countdown(self, instance_attr_time):
        t = 0
        while not t == instance_attr_time:
            sleep(1)
            self.timer = t
            print(self.timer)
            t += 1
            if t == 0:
                self.trigger = True
        print(f' done {instance_attr_time} {self.trigger}')
        return self.trigger


class GIni(GBase):

    ini_keys = {}        # cls attribute to store selections from ini file, works because of key[key] = value, else not
    srv_param_dict = {}  # all ini keys plus short url, suffix, server type stuff
    song_dict = {}       # recorder reads song title from key, related to the url in settings.ini
    start_stop_recording = {}  # ini key: 'start' , 'stop'; while loop check start, working check stop go upper while
    # ini_key + '_single_title', ini_key + '_rec_from_here'
    cost_current_ini = ''  # ini key for cost_dict calc
    cost_dict = {}        # stores len of received headers to calc amount of data searching strings per day
    fail_meta_dict = {}  # can not read metadata from stream, no data
    # list of search strings delimiter blank, first key is named 'STRINGS': Britney Phantom ไม่เคยจะจำ Elton Jim techno
    search_dict = {'STRINGS': 'Britney Spears ไม่เคยจะจำ Elton AC/DC techno Band feat. mix'}  # only show it is working
    list_items = []
    search_title_keys_list = []  # radio short keys, not start recording all streams, only searched titles

    def __init__(self):
        super().__init__()

    @staticmethod
    def show_items_ini_file():

        config = configparser.ConfigParser()  # imported library to wok with .ini files
        try:
            config.read_file(open(GBase.settings_path))
        except FileNotFoundError as ex:
            print(ex)
            sys.exit()
        else:

            i = 0

            print('\t  _______       __  __       ___                      __       ')
            print('\t / ___/ /  ___ / /_/ /____  / _ \___ _______  _______/ /__ ____')
            print('\t/ (_ / _ \/ -_) __/ __/ _ \/ , _/ -_) __/ _ \/ __/ _  / -_) __/')
            print('\t\___/_//_/\__/\__/\__/\___/_/|_|\__/\__/\___/_/  \_,_/\__/_/   ')
            print('                                                       Elvis lebt')
            for _ in dict(config.items('STATIONS')):
                GIni.list_items.append(_)
            for _ in GIni.list_items:
                print('\t>> %-20s <<' % _)

            print(' \n Radio stations in your list. ')
        return

    @staticmethod
    def find_ini_file(key_name):
        config = configparser.ConfigParser()  # imported library to work with .ini files
        try:
            config.read_file(open(GBase.settings_path))
        except FileNotFoundError as ex:
            print(ex)
            sys.exit()
        else:

            station_value = 'False'  # init var for return value
            stations = config['STATIONS']

            try:
                station_value = stations[key_name]  # if all ok get a valid string of value -> True
            except KeyError as ex:
                print(ex)
                print('-- FALSE radio station, input was ->>> ' + key_name)

        return station_value  # single value

    @staticmethod
    def find_all_in_stations():

        config = configparser.ConfigParser()  # imported library to work with .ini files
        try:
            config.read_file(open(GBase.settings_path))
        except FileNotFoundError as ex:
            print(ex)
            sys.exit()
        else:
            stations = config['STATIONS']

        return stations  # list of values

    @staticmethod
    def parse_url_simple_url(radio_url):
        url = radio_url  # whole url is used for connection to radio server

        # 'http:' is first [0], 'ip:port/xxx/yyy' second item [1] in list_url_protocol
        list_url_protocol = url.split("//")
        list_url_ip_port = list_url_protocol[1].split("/")  # 'ip:port' is first item in list_url_ip_port
        radio_simple_url = list_url_protocol[0] + '//' + list_url_ip_port[0]
        return radio_simple_url


class GNet(GBase):
    
    is_shoutcast_server = False
    is_icecast_server = False
    is_icecast_json = False
    is_unknown_server = False
    is_no_meta_data_avail = False
    
    http_pool = urllib3.PoolManager(num_pools=200)
    request = ''
    ua_standard = {'Icy-MetaData': '1', 'user-agent': "Mozilla/5.0 (X11; Linux x86_64) AppleWebKit/537.36 "
                      "(KHTML, like Gecko) Chrome/52.0.2743.82 Safari/537.36"}
    query_shoutcast = '/currentsong'

    def __init__(self):
        super().__init__()

    @staticmethod
    def reset_switch():
        GNet.is_shoutcast_server = False
        GNet.is_icecast_server = False
        GNet.is_icecast_json = False
        GNet.is_unknown_server = False
        GNet.is_no_meta_data_avail = False

    @staticmethod
    def playlist_server(url):

        play_list_server = ''
        if url[-5:] == '.m3u8' or url[-5:] == '.xspf':
            print(' .m3u8/.xspf play lists not yet supported')
            sys.exit()

        if url[-4:] == '.m3u' or url[-4:] == '.pls':  # or url[-5:] == '.m3u8' or url[-5:] == '.xspf':
            response = GNet.http_pool.request('GET', url, preload_content=False)

            response_data_split_by_line = response.read().decode("utf-8").splitlines()
            print(" \n        --- | ---")

            for _ in response_data_split_by_line:
                try:
                    print(_)
                except UnicodeEncodeError:
                    pass
            print(" \n        --- | ---")

            try:
                play_list_server = input('copy and paste a server from above, here ->: ')
            except Exception as ex:
                print(ex)
                print(' exception: input error ')
                return False

        else:
            return False

        return play_list_server.strip()
 
    @staticmethod
    def is_shout_ice_unknown_server(server_port, full_url):
        
        response = GNet.is_server_alive(server_port)
        if not response:
            print(' is_server_alive ' + server_port + 'Can not get the server. Exit')
            return False
        else:
            GNet.is_server_shoutcast(server_port)
            if GNet.is_shoutcast_server:
                GNet.is_shoutcast_server = True
                return GNet.is_shoutcast_server
            if not GNet.is_shoutcast_server:
                GNet.is_server_icecast(server_port)
                if GNet.is_icecast_server:
                    if GNet.is_server_icecast_status_json(server_port, full_url):
                        # http://stream.dancewave.online:8080/retrodance.mp3/status-json.xsl full url
                        GNet.is_icecast_json = True
                        return GNet.is_icecast_json

            if not GNet.is_icecast_json:
                GNet.is_unknown_server = True
                req = GNet.http_pool.request('GET', full_url, headers={'Icy-MetaData': '1'}, preload_content=False)
                try:
                    GNet.icy_metadata = int(req.headers['icy-metaint'])
                    print(f' chunk size {GNet.icy_metadata}')
                    GIni.cost_dict[GIni.cost_current_ini] = GNet.icy_metadata
                    # chunk size, calculate request one day (title search)
                    # write in dict for no metadata = true
                except Exception as ex:
                    print(ex)
                    print(' server has no metadata - try record into one file ')
                    GNet.is_no_meta_data_avail = True
                    return GNet.is_no_meta_data_avail
                return GNet.is_unknown_server
        return

    @staticmethod
    def is_server_meta_stream(url):
        # print('\n(  ( ((GR)) )  ) \t' + url + ' Click the url to listen.')
        # meta info header, if content-type is found do not go tu suffix check
        try:
            print(url)
            GNet.request = GNet.http_pool.request('GET', url, headers=GNet.ua_standard, preload_content=False)

        except Exception as ke:
            print('   --> server failed, no recording ' + url)
            print(ke)
            return False
        else:
            if GNet.request.status != 200:
                GNet.request.release_conn()
                return False
            return True

    @staticmethod
    def is_server_alive(server_name):  # if interc. w. is_server_meta_stream d_kultur swr2_streamuse fail to get name?
        # print(' is_server_alive {server_name}')
        try:
            response = GNet.http_pool.request('GET', server_name, headers=GNet.ua_standard, preload_content=False)
            print(f' response status: {response.status} {server_name}')
        except Exception as ex:
            print(repr(ex))  # zombie server
            return False
        else:
            return True

    @staticmethod
    def is_server_shoutcast(server_name):

        try:
            request = GNet.http_pool.request('GET', server_name, preload_content=False, retries=10)  # winamp
            if not request.headers['icy-notice1'].find('http://www.winamp.com') == -1:  # find() -1 fail
                GNet.is_shoutcast_server = True
                GIni.cost_dict[GIni.cost_current_ini] = len(request.headers)
        except KeyError:
            # print(repr(ex))
            # print(' no Shoutcast stream')
            return False
        else:
            return True

    @staticmethod
    def is_server_icecast(server_name):

        try:
            request = GNet.http_pool.request('GET', server_name, preload_content=False)  # 'server'
            substr_start_num = request.headers['server'].lower().find("icecast")
            if not substr_start_num == -1:  # -1 false from find()
                GNet.is_icecast_server = True
                return True
            else:
                print(f' server type: {request.headers["server"]}')
        except KeyError:
            # print(repr(ex))
            # print(' Exception in is_server_icecast ')
            return False

    @staticmethod
    def ice_status_json(request):
        data_stats_source_title = False
        source = request.read()
        data = json.loads(source)

        data_stats = data.get('icestats')
        data_stats_source = data_stats.get('source')
        try:
            data_stats_source_title = data_stats_source.get('title')
        except AttributeError:
            # print(repr(e))
            data_stats_source = data_stats.get('source')
            data_stats_source_title = data_stats_source[0]['title']

        GIni.cost_dict[GIni.cost_current_ini] = len(source)
        return data_stats_source_title

    @staticmethod
    def is_server_icecast_status_json(server_name, full_url):

        try:
            request = GNet.http_pool.request('GET', server_name + '/status-json.xsl', preload_content=False)
            #print(f' 1is_server_icecast_status_json ')
            song_title = GNet.ice_status_json(request)
            #print(f'2is_server_icecast_status_json  {song_title}')
            GNet.is_icecast_server = True
            GNet.is_icecast_json = True
        
        except Exception as ex:
            print(ex)
            print(' server got no readable json file \n will read from stream directly \n')
            # GNet.is_unknown_server = True
            return False
        else:
            return song_title

    @staticmethod
    def stream_filetype(server_name):

        request = GNet.http_pool.request('GET', server_name, headers=GNet.ua_standard, preload_content=False)
        content_type = ''
        try:
            request.headers
        except Exception as ex:
            print(ex)
            print(' stream_filetype in EXCEPTION')
        else:
            if request.headers['content-type'] == 'audio/aacp' or request.headers['content-type'] == 'application/aacp':
                content_type = '.aacp'
            if request.headers['content-type'] == 'audio/aac':
                content_type = '.aac'
            if request.headers['content-type'] == 'audio/ogg' or request.headers['content-type'] == 'application/ogg':
                content_type = '.ogg'
            if request.headers['content-type'] == 'audio/mpeg':
                content_type = '.mp3'
            if request.headers['content-type'] == 'audio/x-mpegurl' or request.headers['content-type'] == 'text/html':
                content_type = '.m3u'
            # application/x-winamp-playlist , audio/scpls , audio/x-scpls ,  audio/x-mpegurl

            return content_type


class GRecorder:

    stream_song_name = '_no-name-record_no_split_'
    path_to_song_dict = {}  # each thread writes the new title to the station key name {station : title}

    def __init__(self, url, ini_key):
        self.url = url
        self.ini_key = ini_key

    @staticmethod
    def thread_pull_song_name(url, ini_key, params_dict, path_to_song_dict):
        # print(f'=== thread_pull_song_name ... ')
        update_terminal = False
        stream_song_name = GRecorder.path_to_song_dict[ini_key]
        GIni.fail_meta_dict[url] = 'False'

        while not GBase.exit_app:

            if GIni.srv_param_dict[ini_key + '_is_unknown_server'] == 'True':
                if not GIni.fail_meta_dict[url] == 'True':  # let record go in one file
                    ret = GRecorder.get_metadata_from_stream(url)  # read one time from stream
                    stream_song_name = ret

            if GIni.srv_param_dict[ini_key + '_is_icecast_json'] == 'True':
                ice_short_url = GIni.srv_param_dict[url]  # get http://ip:port from full url
                # ice_short_url = GIni.parse_url_simple_url(url)
                stream_song_name = GNet.is_server_icecast_status_json(ice_short_url, ice_short_url)

            if GIni.srv_param_dict[ini_key + '_is_shoutcast_server'] == 'True':
                srv = GIni.srv_param_dict[url]  # get http://ip:port from full url
                srv_current = srv + '/currentsong'
                response = GNet.http_pool.request('GET', srv_current, headers=GNet.ua_standard, preload_content=False)
                stream_song_name = response.data.decode('utf-8')

            if GIni.srv_param_dict[ini_key + '_is_no_meta_data_avail'] == 'True':
                stream_song_name = '_no-name-record_no_split_'
                # print(' _is_no_meta_data_avail ' + stream_song_name)

            if not len(stream_song_name) <= 2:
                GRecorder.path_to_song_dict[ini_key] = stream_song_name
                # print(f' <<<<<<<<<<<<<<<{ini_key} GRecorder.path_to_song_dict: {GRecorder.path_to_song_dict}')
                if not update_terminal == stream_song_name:
                    update_terminal = stream_song_name
                    print('\t\t title on ' + ini_key + ':\t' + update_terminal)
            # give recorder command to record, we got a match from search
            GIni.search_title_keys_list.append(42)
            try:
                for _ in GIni.search_title_keys_list:
                    if _ == ini_key:
                        # if some phrases are found in def search_pattern_start_record
                        got_it = GRecorder.search_pattern_start_record(stream_song_name, ini_key)
                        if got_it:
                            GIni.start_stop_recording[ini_key] = 'start'
                        else:
                            GIni.start_stop_recording[ini_key] = 'stop'

            except Exception as ex:
                print(ex)
            # GRecorder.search_pattern_start_record(stream_song_name, ini_key)
            # print(GIni.start_stop_recording)
            sleep(GBase.sleeper)

            if GBase.exit_app:
                print(f' stop get title {ini_key}')

    @staticmethod
    def search_pattern_start_record(title, ini_key):
        # search title for strings to see if we should start recording this song/title
        search_list = []  # chop the string
        try:
            strings = GIni.search_dict[ini_key]  # dict with user search strings
            # print(f' strings; {strings} {ini_key}' )
            search_list = strings.encode('utf-8').lower().split(b' ')
        except KeyError as ex:
            pass

        for search_str in search_list:

            if not len(search_str) <= 1:  # b'' match problem

                if not title.encode('utf-8').lower().find(search_str) == -1:  # find() returns -1, if not found
                    print(f'<<< match station: {ini_key} phrase: {search_str}')
                    # GIni.start_stop_recording[ini_key + '_adv'] = 'start_from_here'
                    return True
        return False

    @staticmethod
    def record_songs(url, directory_save, stream_suffix, radio_short_key, path_to_song_dict):
        # print('record_songs')
        stream_request_size = io.DEFAULT_BUFFER_SIZE
        fresh_song = 'False'
        old_time = time()
        sleeper_recorder = False
        try:
            if GIni.start_stop_recording[radio_short_key] == 'stop':
                sleeper_recorder = True
        except KeyError:
            pass

        sleep(GBase.sleeper + 1)  # wait to get song name from def: thread_pull_song_name
        stream_song_name = GRecorder.path_to_song_dict[radio_short_key]

        try:
            request = GNet.http_pool.request('GET', url, preload_content=False, retries=10)
        except urllib3.exceptions.NewConnectionError:
            print('Connection failed. ' + url)
        else:
            # print(' \n Recording ... ')
            request.auto_close = False  # NOT DEL !!! :)

            while not GBase.exit_app:
                while not GBase.exit_app:
                    if GIni.start_stop_recording[radio_short_key] == 'stop':
                        sleep(.1)
                    else:
                        # print(f'----> else {radio_short_key} sleeper_recorder {sleeper_recorder}'
                        #      f' dict {GIni.start_stop_recording[radio_short_key]}')
                        # recorder def loose connect if to long idle
                        if sleeper_recorder:
                            print(f'----> start record: {radio_short_key}')
                            try:
                                request = GNet.http_pool.request('GET', url, preload_content=False, retries=10)
                            except urllib3.exceptions.NewConnectionError:
                                pass
                        break
                i = 0
                while not GBase.exit_app:  # outer loop create empty new files
                    if GIni.start_stop_recording[radio_short_key] == 'stop':
                        print(f'----> stop record: {radio_short_key}')

                        break

                    if not fresh_song == stream_song_name:

                        fresh_song = stream_song_name
                        clean_name = GBase.remove_special_chars(stream_song_name)

                        if clean_name == '_no-name-record_no_split_':
                            clean_name = clean_name + GBase.this_time()
                            clean_name = GBase.remove_special_chars(clean_name)

                        fresh_file = directory_save + '//' + clean_name + stream_suffix
                        if i == 5:
                            print(f' \t\t\t\t\t\t(  (  ( (Ghetto Recorder) )  )  ) )')
                            i = 0
                        i += 1
                        with open(fresh_file, 'wb') as record_file:

                            while not GBase.exit_app:  # inner loop get the stream and write into file
                                chunk = request.read(stream_request_size)  # wasting no space in cluster size
                                record_file.write(chunk)
                                # this_chunk_len = int(len(chunk))
                                # result = result + this_chunk_len

                                if time() - old_time > 60:
                                    old_time = time()
                                    # print(f' {radio_short_key} mb/min: {round((result / 1024) / 1024, 2)} mb')
                                    # result = 0
                                if not chunk:
                                    record_file.flush()
                                    break

                                stream_song_name_new = GRecorder.path_to_song_dict[radio_short_key]  # look what's new

                                if not stream_song_name_new == '':
                                    stream_song_name = stream_song_name_new
                                if not fresh_song == stream_song_name:  # only here break, jump to outer loop
                                    record_file.flush()
                                    break

                            if GBase.exit_app:
                                print(f' exit record {radio_short_key}')
                                break
        return

    @staticmethod
    def get_metadata_from_stream(url):

        try:
            request = GNet.http_pool.request('GET', url, headers={'Icy-MetaData': '1'}, preload_content=False)

            icy_metadata = request.headers['icy-metaint']
            icy_metadata = int(icy_metadata)
        except KeyError as ke:
            print(ke)
            print('Connection failed miserable in get_metadata_from_stream reader: ' + url)
            GIni.fail_meta_dict[url] = 'True'
        else:
            request.read(icy_metadata)
            chunk_1b = request.read(1)
            chunk_1b = ord(chunk_1b)
            # \x03 hex value, was div. by 4; hex C (12d) * hex 4 = 30 48d; hex 3 (3d) * 16d = 48d;
            read_bytes = chunk_1b * 16
            read_bytes = int(read_bytes)
            metadata_content = request.read(read_bytes)
            metadata_content = metadata_content.decode('utf-8')
            title_info = metadata_content.split(";")
            title_info = title_info[0].split("=")
            title_info = GBase.remove_special_chars(title_info[1])
            request.release_conn()
            return title_info


def step1_collect_stream_server():
    # collect the keys from settings.ini, test if server is alive
    # produce a dictionary with key: short name, value: url

    GIni.show_items_ini_file()  # print to terminal
    valid_input = False
    while True:  # collect a list of radio keys to work on
        ini_file_input = input('Copy/Paste a Radio >> settings.ini <<, Enter to record -->:')
        if ini_file_input == 42:
            valid_input = True
            # read all keys from ini file

        if ini_file_input == '':  # collect valid keys till enter
            break
        else:
            valid_input = True
            print(f' Hit Enter <---| to RECORD, or paste next radio, paste 42 for ALL radios ')
            str_key = ini_file_input.strip()

            if str_key:
                str_val = GIni.find_ini_file(GBase.remove_special_chars(str_key))

                if str_key == '42':
                    print('<<<<<<<<<<<<<<<<<<<<<<<<<<<<<< 42 >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>')
                    # if 42, return gives a list instead of a value
                    str_list = GIni.find_all_in_stations()
                    for ini_key in str_list:
                        ini_row_value = str_list[ini_key]
                        add_server_to_data_base(ini_key, ini_row_value)
                else:

                    add_server_to_data_base(str_key, str_val)  # work with one value

    if valid_input:
        step2_test_stream_server(GIni.ini_keys)
    else:
        print(f' not input - exit')


def add_server_to_data_base(str_key, str_val):
    GIni.ini_keys[str_key] = str_val  # append url to dictionary as value
    GBase.make_directory(GBase.radio_base_dir)
    GBase.make_directory(GBase.radio_base_dir + '//' + str_key)

    is_playlist_server = GNet.playlist_server(GIni.ini_keys[str_key])
    if is_playlist_server:  # update dictionary with new url
        GIni.ini_keys[str_key] = is_playlist_server  # append dictionary, test if it is alive
        if not GNet.is_server_meta_stream(GIni.ini_keys[str_key]):
            print('   --> playlist_server server failed, no recording')
            del GIni.ini_keys[str_key]
    if not GNet.is_server_meta_stream(GIni.ini_keys[str_key]):  # first time internet access, response code
        # delete key from dict, return
        del GIni.ini_keys[str_key]


def step2_test_stream_server(ini_keys):
    # further update dictionary
    for key in ini_keys:
        test_stream_server(key)
    # print(GIni.srv_param_dict)
    # calc_connection_cost()
    record(ini_keys, GIni.srv_param_dict)  # this loops at end for signal


def test_stream_server(key):
    ini_keys = GIni.ini_keys
    GIni.cost_current_ini = key  # for Byte cost calculation

    # write next dictionary with connection parameters
    GIni.srv_param_dict[key] = ini_keys[key]
    # print(f'   GIni.srv_param_dict[key]   {GIni.srv_param_dict[key]}    {path_dict[key]}')

    GNet.reset_switch()  # those GNet class vars are modified each loop
    GIni.srv_param_dict[ini_keys[key]] = GIni.parse_url_simple_url(GIni.srv_param_dict[key])  # http://ip:port

    # str_in is input, modify to get more dictionary keys; file type, is shout is ice ...
    ret = GNet.stream_filetype(GIni.srv_param_dict[key])
    GIni.srv_param_dict[key + '_file'] = GNet.stream_filetype(GIni.srv_param_dict[key])
    if ret is None or ret == '' or ret == ' ':
        GIni.srv_param_dict[key + '_file'] = '.mp3'
    # set switch to server type to either read the title from http or from http stream, GNet class vars mod.
    GNet.is_shout_ice_unknown_server(GIni.srv_param_dict[ini_keys[key]], GIni.srv_param_dict[key])
    if GNet.is_shoutcast_server:
        GIni.srv_param_dict[key + '_is_shoutcast_server'] = 'True'
    else:
        GIni.srv_param_dict[key + '_is_shoutcast_server'] = 'False'
    if GNet.is_icecast_server:
        GIni.srv_param_dict[key + '_is_icecast_server'] = 'True'
    else:
        GIni.srv_param_dict[key + '_is_icecast_server'] = 'False'
    if GNet.is_icecast_json:
        GIni.srv_param_dict[key + '_is_icecast_json'] = 'True'
    else:
        GIni.srv_param_dict[key + '_is_icecast_json'] = 'False'
    if GNet.is_unknown_server:
        GIni.srv_param_dict[key + '_is_unknown_server'] = 'True'
    else:
        GIni.srv_param_dict[key + '_is_unknown_server'] = 'False'
    if GNet.is_no_meta_data_avail:
        GIni.srv_param_dict[key + '_is_no_meta_data_avail'] = 'True'
    else:
        GIni.srv_param_dict[key + '_is_no_meta_data_avail'] = 'False'


def calc_connection_cost():

    for item in GIni.cost_dict:
        print(f' key: {item} Bytes: {GIni.cost_dict[item]}, title search only - cost per/hour: '
              f'{round(((int(GIni.cost_dict[item]) * 30 * 60)/1024) /1024, 2)}mb')


def print_stream_server():

    print(f'GIni.srv_param_dict {GIni.srv_param_dict}')


def record(ini_keys, path_dict_srv_params):
    signal.signal(signal.SIGTERM, signal_handler)
    signal.signal(signal.SIGINT, signal_handler)

    for ini_key in ini_keys:

        GIni.song_dict[ini_key] = '_no-name-record_no_split_'  # init the dict for this thread
        GIni.start_stop_recording[ini_key] = 'start'
        # GIni.start_stop_recording[ini_key] = 'stop'            # init it here, should be set via user interface
        GIni.start_stop_recording[ini_key + '_adv'] = 'start_from_here'  # MUST be set or a key error in record def:

        url = GIni.ini_keys[ini_key]
        GRecorder.path_to_song_dict = GIni.song_dict
        # print(f' <<<<<<<<<<<<<<< GRecorder.path_to_song_dict: {GRecorder.path_to_song_dict}')

        GBase.pool.submit(GRecorder.thread_pull_song_name, url, ini_key, path_dict_srv_params, None)

        dir_save = GBase.radio_base_dir + '//' + ini_key
        stream_suffix = path_dict_srv_params[ini_key + '_file']
        
        GBase.pool.submit(GRecorder.record_songs, url, dir_save, stream_suffix, ini_key, None)

    while not GBase.exit_app:
        sleep(1)


def main():
    step1_collect_stream_server()


if __name__ == '__main__':
    main()
